@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/child.c,v 1.22 2008/05/08 03:02:04 y-asaba Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/child.c,v 1.26 2009/01/22 00:50:26 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2008	PgPool Global Development Group
+ * Copyright (c) 2003-2009	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -57,12 +57,10 @@
 #include "pool_ip.h"
 #include "md5.h"
 
-void child_exit(int code);
-
 static POOL_CONNECTION *do_accept(int unix_fd, int inet_fd, struct timeval *timeout);
 static StartupPacket *read_startup_packet(POOL_CONNECTION *cp);
 static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION *frontend);
-static void cancel_request(CancelPacket *sp, int secondary_backend);
+static void cancel_request(CancelPacket *sp);
 static RETSIGTYPE die(int sig);
 static RETSIGTYPE close_idle_connection(int sig);
 static RETSIGTYPE wakeup_handler(int sig);
@@ -226,9 +224,8 @@ void do_child(int unix_fd, int inet_fd)
 		/* cancel request? */
 		if (sp->major == 1234 && sp->minor == 5678)
 		{
-			cancel_request((CancelPacket *)sp->startup_packet, 0);
-			if (DUAL_MODE)
-				cancel_request((CancelPacket *)sp->startup_packet, 1);
+			cancel_request((CancelPacket *)sp->startup_packet);
+
 			pool_close(frontend);
 			pool_free_startup_packet(sp);
 			connection_count_down();
@@ -436,11 +433,12 @@ void do_child(int unix_fd, int inet_fd)
 					/*
 					 * do not cache connection if:
 					 * pool_config->connection_cahe == 0 or
-					 * datase name is template0, template1, or regression
+					 * database name is template0, template1, postgres or regression
 					 */
 					if (pool_config->connection_cache == 0 ||
 						!strcmp(sp->database, "template0") ||
 						!strcmp(sp->database, "template1") ||
+						!strcmp(sp->database, "postgres") ||
 						!strcmp(sp->database, "regression"))
 					{
 						pool_close(frontend);
@@ -1077,16 +1075,33 @@ int send_startup_packet(POOL_CONNECTION_POOL_SLOT *cp)
 /*
  * process cancel request
  */
-static void cancel_request(CancelPacket *sp, int backend_id)
+static void cancel_request(CancelPacket *sp)
 {
 	int	len;
 	int fd;
 	POOL_CONNECTION *con;
 	int i;
+	ConnectionInfo *c = NULL;
+	CancelPacket cp;
 
 	pool_debug("Cancel request received");
 
-	for (i=0;i<NUM_BACKENDS;i++)
+	/* look for cancel key from shmem info */
+	for (i=0;i<pool_config->num_init_children*pool_config->max_pool;i++)
+	{
+		c = &con_info[i];
+
+		if (c->pid == sp->pid && c->key == sp->key)
+		{
+			pool_debug("found pid:%d key:%d i:%d",c->pid, c->key,i);
+			c = &con_info[i/pool_config->max_pool * pool_config->max_pool];
+			break;
+		}
+	}
+	if (i == pool_config->num_init_children*pool_config->max_pool)
+		return;	/* invalid key */
+	
+	for (i=0;i<NUM_BACKENDS;i++,c++)
 	{
 		if (!VALID_BACKEND(i))
 			continue;
@@ -1109,10 +1124,22 @@ static void cancel_request(CancelPacket *sp, int backend_id)
 		len = htonl(sizeof(len) + sizeof(CancelPacket));
 		pool_write(con, &len, sizeof(len));
 
-		if (pool_write_and_flush(con, sp, sizeof(CancelPacket)) < 0)
+		cp.protoVersion = sp->protoVersion;
+		cp.pid = c->pid;
+		cp.key = c->key;
+
+		pool_debug("pid:%d key: %d",cp.pid,cp.key);
+
+		if (pool_write_and_flush(con, &cp, sizeof(CancelPacket)) < 0)
 			pool_error("Could not send cancel request packet for backend %d", i);
 
 		pool_close(con);
+
+		/*
+		 * this is needed to enure that the next DB node executes the
+		 * query supposed to be canceled.
+		 */
+		sleep(1);
 	}
 }
 
@@ -1258,8 +1285,9 @@ static RETSIGTYPE authentication_timeout(int sig)
 }
 
 /*
- * send frontend exiting messages to all connections.
- * this is called when child life time expires or child max connections expires.
+ * send frontend exiting messages to all connections.  this is called
+ * in any case when child process exits, for example failover, child
+ * life time expires or child max connections expires.
  */
 static void send_frontend_exits(void)
 {
@@ -1325,11 +1353,16 @@ void pool_free_startup_packet(StartupPacket *sp)
 	}
 }
 
+/*
+ * Do house keeping works when pgpool child process exits
+ */
 void child_exit(int code)
 {
+	/* count down global connection counter */
 	if (accepted)
 		connection_count_down();
 
+	/* prepare to shutdown connections to system db */
 	if(pool_config->parallel_mode || pool_config->enable_query_cache)
 	{
 		if (system_db_info->pgconn)
@@ -1337,6 +1370,10 @@ void child_exit(int code)
 		if (pool_system_db_connection())
 			pool_close(pool_system_db_connection()->con);
 	}
+
+	/* let backend know now we are exiting */
+	send_frontend_exits();
+
 	exit(code);
 }
 
