@@ -1,12 +1,12 @@
 /* -*-pgsql-c-*- */
 /*
  *
- * $Header: /cvsroot/pgpool/pgpool-II/pool.h,v 1.25 2007/11/14 03:59:58 y-mori Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pool.h,v 1.36 2009/01/25 10:13:15 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2007	PgPool Global Development Group
+ * Copyright (c) 2003-2009	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -33,6 +33,7 @@
 #include <stdio.h>
 #include <time.h>
 #include <sys/types.h>
+#include <limits.h>
 
 /* undef this if you have problems with non blocking accept() */
 #define NONE_BLOCK
@@ -55,7 +56,7 @@
 #define DEFAULT_SOCKET_DIR "/tmp"
 
 /* pid file name */
-#define PID_FILE_NAME "pgpool.pid"
+#define DEFAULT_PID_FILE_NAME "/var/run/pgpool/pgpool.pid"
 
 typedef enum {
 	POOL_CONTINUE = 0,
@@ -133,22 +134,14 @@ typedef struct {
 	int authentication_timeout; /* maximum time in seconds to complete client authentication */
     int	max_pool;	/* max # of connection pool per child */
     char *logdir;		/* logging directory */
+    char *pid_file_name;		/* pid file name */
     char *backend_socket_dir;	/* Unix domain socket directory for the PostgreSQL server */
 	int replication_mode;		/* replication mode */
-	int replication_strict;	/* if non 0, wait for completion of the
-                               query sent to master to avoid deadlock */
 
 	int log_connections;		/* 0:false, 1:true - logs incoming connections */
 	int log_hostname;		/* 0:false, 1:true - resolve hostname */
 	int enable_pool_hba;		/* 0:false, 1:true - enables pool_hba.conf file authentication */
 
-	/*
-	 * if next PostgreSQL server does not respond in this milli
-	 * seconds, abort this session.  this is not compatible with
-	 * replication_strict = 1. 0 means no timeout.
-	 */
-	int replication_timeout;
-
 	int load_balance_mode;		/* load balance mode */
 
 	int replication_stop_on_mismatch;		/* if there's a data mismatch between master and secondary
@@ -169,6 +162,10 @@ typedef struct {
 	char *recovery_password;		/* PostgreSQL user password for online recovery */
 	char *recovery_1st_stage_command;   /* Online recovery command in 1st stage */
 	char *recovery_2nd_stage_command;   /* Online recovery command in 2nd stage */
+	int recovery_timeout;				/* maximum time in seconds to wait for remote start-up */
+	int client_idle_limit_in_recovery;		/* If > 0, the client is forced to be
+											 *  disconnected after n seconds idle
+											 *  This parameter is only valid while in recovery 2nd statge */
 	int insert_lock;	/* if non 0, automatically lock table with INSERT to keep SERIAL
 						   data consistency */
 	int ignore_leading_white_space;		/* ignore leading white spaces of each query */
@@ -274,7 +271,7 @@ typedef struct {
 } POOL_CONNECTION_POOL_SLOT;
 
 typedef struct {
-	ConnectionInfo *info;
+	ConnectionInfo *info;		/* connection info on shmem */
     POOL_CONNECTION_POOL_SLOT	*slots[MAX_NUM_BACKENDS];
 } POOL_CONNECTION_POOL;
 
@@ -314,15 +311,16 @@ typedef struct {
 					  (((!REPLICATION && !PARALLEL_MODE)||master_slave_dml)? Req_info->master_node_id+1: \
 					   pool_config->backend_desc->num_backends))
 #endif
-#define NUM_BACKENDS (in_load_balance? (selected_slot+1) : \
-					  (((!REPLICATION && !PARALLEL_MODE) && !MASTER_SLAVE)? Req_info->master_node_id+1: \
-					   pool_config->backend_desc->num_backends))
+/* NUM_BACKENDS now always returns actual number of backends if not in_load_balance */
+#define NUM_BACKENDS (in_load_balance ? (selected_slot+1) : pool_config->backend_desc->num_backends)
 #define BACKEND_INFO(backend_id) (pool_config->backend_desc->backend_info[(backend_id)])
 #define LOAD_BALANCE_STATUS(backend_id) (pool_config->load_balance_status[(backend_id)])
+/* if RAW_MODE, VALID_BACKEND returns the selected node only */
 #define VALID_BACKEND(backend_id) \
+	(RAW_MODE ? (backend_id) == MASTER_NODE_ID : \
 	(in_load_balance ? LOAD_BALANCE_STATUS(backend_id) == LOAD_SELECTED : \
     ((BACKEND_INFO(backend_id).backend_status == CON_UP) || \
-	 (BACKEND_INFO(backend_id).backend_status == CON_CONNECT_WAIT)))
+	 (BACKEND_INFO(backend_id).backend_status == CON_CONNECT_WAIT))))
 #define CONNECTION_SLOT(p, slot) ((p)->slots[(slot)])
 #define CONNECTION(p, slot) (CONNECTION_SLOT(p, slot)->con)
 #define MASTER_CONNECTION(p) ((p)->slots[MASTER_NODE_ID])
@@ -333,6 +331,7 @@ typedef struct {
 #define MASTER_SLAVE (pool_config->master_slave_enabled)
 #define DUAL_MODE (REPLICATION || MASTER_SLAVE)
 #define PARALLEL_MODE (pool_config->parallel_mode)
+#define RAW_MODE (!REPLICATION && !PARALLEL_MODE && !MASTER_SLAVE)
 #define MASTER(p) MASTER_CONNECTION(p)->con
 //#define SECONDARY(p) SECONDARY_CONNECTION(p)->con
 #define MAJOR(p) MASTER_CONNECTION(p)->sp->major
@@ -382,6 +381,28 @@ typedef struct {
 	int conn_counter;
 } POOL_REQUEST_INFO;
 
+/* description of row. corresponding to RowDescription message */
+typedef struct {
+	char *attrname;		/* attribute name */
+	int oid;	/* 0 or non 0 if it's a table oblect */
+	int attrnumber;		/* attribute number starting with 1. 0 if it's not a table */
+	int typeoid;		/* data type oid */
+	int	size;	/* data length minus means variable data type */
+	int mod;	/* data type modifier */
+} AttrInfo;
+
+typedef struct {
+	int num_attrs;		/* number of attributes */
+	AttrInfo *attrinfo;
+} RowDesc;
+
+typedef struct {
+	RowDesc *rowdesc;	/* attribute info */
+	int numrows;		/* number of rows */
+	int *nullflags;	/* if NULL, -1 or length of the string excluding termination null */
+	char **data;		/* actual row character data terminated with null */
+} POOL_SELECT_RESULT;
+
 /*
  * global variables
  */
@@ -392,7 +413,8 @@ extern volatile sig_atomic_t backend_timer_expired; /* flag for connection close
 extern long int weight_master;	/* normalized weight of master (0-RAND_MAX range) */
 extern int my_proc_id;  /* process table id (!= UNIX's PID) */
 extern POOL_SYSTEMDB_CONNECTION_POOL *system_db_info; /* systemdb */
-extern ProcessInfo *pids; /* process information table */
+extern ProcessInfo *pids; /* shmem process information table */
+extern ConnectionInfo *con_info; /* shmem connection info table */
 extern int in_load_balance;		/* non 0 if in load balance mode */
 extern int selected_slot;		/* selected DB node for load balance */
 extern int master_slave_dml;	/* non 0 if master/slave mode is specified in config file */
@@ -401,6 +423,9 @@ extern volatile sig_atomic_t *InRecovery;
 extern char remote_ps_data[];		/* used for set_ps_display */
 extern volatile sig_atomic_t got_sighup;
 
+#define QUERY_STRING_BUFFER_LEN 1024
+extern char query_string_buffer[];		/* last query string sent to simpleQuery() */
+
 /*
  * public functions
  */
@@ -423,7 +448,6 @@ extern POOL_STATUS pool_process_query(POOL_CONNECTION *frontend,
 extern POOL_CONNECTION *pool_open(int fd);
 extern void pool_close(POOL_CONNECTION *cp);
 extern int pool_read(POOL_CONNECTION *cp, void *buf, int len);
-extern int pool_read_parallel(POOL_CONNECTION *cp, void *buf, int len);
 extern char *pool_read2(POOL_CONNECTION *cp, int len);
 extern int pool_write(POOL_CONNECTION *cp, void *buf, int len);
 extern int pool_flush(POOL_CONNECTION *cp);
@@ -459,9 +483,7 @@ extern int connect_unix_domain_socket(int secondary_backend);
 extern int connect_inet_domain_socket_by_port(char *host, int port);
 extern int connect_unix_domain_socket_by_port(int port, char *socket_dir);
 
-extern int pool_check_fd(POOL_CONNECTION *cp, int notimeout);
-extern void pool_enable_timeout(void);
-extern void pool_disable_timeout(void);
+extern int pool_check_fd(POOL_CONNECTION *cp);
 
 extern void pool_send_frontend_exits(POOL_CONNECTION_POOL *backend);
 
@@ -518,6 +540,8 @@ extern POOL_STATUS OneNode_do_command(POOL_CONNECTION *frontend, POOL_CONNECTION
 extern POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
 	char *hostname, int port, char *dbname, char *user, char *password);
 
+extern POOL_STATUS do_query(POOL_CONNECTION *backend, char *query, POOL_SELECT_RESULT **result);
+
 /* define pool_system.c */
 extern POOL_CONNECTION_POOL_SLOT *pool_system_db_connection(void);
 extern DistDefInfo *pool_get_dist_def_info (char * dbname, char * schema_name, char * table_name);
